package de.ozgcloud.bescheid;

import java.time.LocalDate;
import java.util.List;
import java.util.Optional;

import de.ozgcloud.bescheid.vorgang.VorgangId;
import de.ozgcloud.common.binaryfile.FileId;
import lombok.Builder;
import lombok.Getter;
import lombok.Singular;

@Builder(toBuilder = true)
@Getter
public class Bescheid {

	public static final String FIELD_VORGANG_ID = "vorgangId";
	public static final String FIELD_CREATED_BY = "createdBy";

	public static final String FIELD_BEWILLIGT = "bewilligt";
	public static final String FIELD_STATUS = "status";
	public static final String FIELD_BESCHEID_DOCUMENT = "bescheidDocument";
	public static final String FIELD_BESCHIEDEN_AM = "beschiedenAm";
	public static final String FIELD_SEND_BY = "sendBy";

	public static final String FIELD_NACHRICHT_TEXT = "nachrichtText";
	public static final String FIELD_NACHRICHT_SUBJECT = "nachrichtSubject";
	public static final String FIELD_ATTACHMENTS = "attachments";
	public static final String FIELD_SENT_INFO = "sentInfo";

	private BescheidId id;
	private long version;

	private VorgangId vorgangId;
	private UserId createdBy;

	private boolean bewilligt;
	private Status status;
	private FileId bescheidDocument;
	private LocalDate beschiedenAm;
	private SendBy sendBy;

	private SentInfo sentInfo;

	@Builder.Default
	private Optional<String> nachrichtSubject = Optional.empty();
	@Builder.Default
	private Optional<String> nachrichtText = Optional.empty();

	@Singular
	private List<FileId> attachments;

	public enum Status {
		DRAFT, SENT;

		public boolean not(Object value) {
			return !hasValue(value);
		}

		public boolean hasValue(Object value) {
			return this.name().equalsIgnoreCase(String.valueOf(value));
		}
	}

	public enum SendBy {
		NACHRICHT, MANUAL;

		public boolean notValue(Object sendByValue) {
			return !hasValue(sendByValue);
		}

		public boolean hasValue(Object sendByValue) {
			return this.name().equalsIgnoreCase(String.valueOf(sendByValue));
		}

	}
}