package de.ozgcloud.bescheid.vorgang;

import java.io.IOException;
import java.util.List;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;

import de.ozgcloud.bescheid.vorgang.FormDataEntry.FormField;
import de.ozgcloud.bescheid.vorgang.FormDataEntry.SubForm;
import de.ozgcloud.common.errorhandling.TechnicalException;

public class FormDataEntrySerializer extends StdSerializer<FormDataEntry> {

	public FormDataEntrySerializer() {
		super(FormDataEntry.class);
	}

	@Override
	public void serialize(FormDataEntry value, JsonGenerator gen, SerializerProvider provider) throws IOException {

		if (value.isFormEntry()) {
			var formEntry = (FormField) value;
			Optional.ofNullable(formEntry.getValue()).map(Object::toString).filter(StringUtils::isNoneBlank)
					.ifPresent(val -> writeStringField(value.getName(), val, gen));
		}
		if (value.isSubForm()) {
			gen.writeFieldName(value.getName());
			gen.writeStartObject();
			var subForm = (SubForm) value;

			subForm.getEntries().forEach(entry -> writeObject(entry, gen));
			gen.writeEndObject();
		}
	}

	private void writeStringField(String name, String value, JsonGenerator gen) {
		try {
			gen.writeStringField(name, value);
		} catch (IOException e) {
			throw new TechnicalException("Error writing string field in json.", e);
		}
	}

	static void writeObject(Object pojo, JsonGenerator jgen) {
		try {
			jgen.writeObject(pojo);
		} catch (IOException e) {
			throw new TechnicalException("Error on writing object to json.", e);
		}
	}
}

class FormDataEntryListSerializer extends StdSerializer<List<FormDataEntry>> {

	public FormDataEntryListSerializer() {
		super(List.class, true);
	}

	@Override
	public void serialize(List<FormDataEntry> value, JsonGenerator gen, SerializerProvider provider) throws IOException {
		gen.writeStartObject();

		value.forEach(entry -> defaultSerializeValue(entry, gen, provider));
		gen.writeEndObject();
	}

	private void defaultSerializeValue(Object value, JsonGenerator jgen, SerializerProvider provider) {
		try {
			var serializer = provider.findValueSerializer(FormDataEntry.class);
			serializer.serialize(value, jgen, provider);
		} catch (IOException e) {
			throw new TechnicalException("Error serializing value.", e);
		}
	}

}
