/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.bescheid.administration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.cloud.endpoint.RefreshEndpoint;

class AdministrationServiceTest {

	@InjectMocks
	private AdministrationService service;

	@Mock
	private AdministrationProperties properties;
	@Mock
	private RefreshEndpoint refreshEndpoint;

	@Nested
	class GetNachrichtSignature {

		@Test
		void shouldCallGetSignature() {
			service.getNachrichtSignature();

			verify(properties).getSignatur();
		}

		@Test
		void shouldReturnSignature() {
			var signature = "signature";
			when(properties.getSignatur()).thenReturn(Optional.of(signature));

			var result = service.getNachrichtSignature();

			assertThat(result).contains(signature);
		}
	}

	@Nested
	class TestRefreshConfig {

		@Test
		void shouldCallRefresh() {
			service.refreshConfig();

			verify(refreshEndpoint).refresh();
		}
	}
}